@@ -125,9 +125,7 @@ class BleakClientWinRT(BaseBleakClient):
 
         # Backend specific. WinRT objects.
         if isinstance(address_or_ble_device, BLEDevice):
-            self._device_info = (
-                address_or_ble_device.address.details.adv.bluetooth_address
-            )
+            self._device_info = address_or_ble_device.details.adv.bluetooth_address
         else:
             self._device_info = None
         self._requester = None
